// -*- c-basic-offset: 2; related-file-name: "printTime.h" -*-
/*
 * @(#)$Id: printWatch.C,v 1.12 2006/06/27 04:47:19 tcondie Exp $
 * 
 * This file is distributed under the terms in the attached LICENSE file.
 * If you do not find this file, copies can be found by writing to:
 * Intel Research Berkeley, 2150 Shattuck Avenue, Suite 1300,
 * Berkeley, CA, 94704.  Attention:  Intel License Inquiry.
 * Or
 * UC Berkeley EECS Computer Science Division, 387 Soda Hall #1776, 
 * Berkeley, CA,  94707. Attention: P2 Group.
 * 
 */

#include "printWatch.h"
#include "p2Time.h"
#include "val_tuple.h"
#include <iostream>
#include <errno.h>

PrintWatch::PrintWatch(string prefix, std::set<string> tableNames, 
		       FILE* output)
  : Element(prefix, 1, 1),
    _prefix(prefix), _output(output)
{
  _tableNames = tableNames;  
}

PrintWatch::PrintWatch(string prefix, std::vector<string> tableNames, 
		       FILE* output)
  : Element(prefix, 1, 1),
    _prefix(prefix), _output(output)
{
  for (std::vector<string>::iterator iter = tableNames.begin();
       iter != tableNames.end(); iter++) {
    _tableNames.insert(*iter);
  }
}


PrintWatch::~PrintWatch()
{
}

TuplePtr PrintWatch::simple_action(TuplePtr p)
{
  if (_tableNames.find((*p)[0]->toString()) == _tableNames.end()) {
    return p; // we don't care about print this
  }

  double bytes = 0;
  for (unsigned int i = 0; i < p->size(); i++) {
    ValuePtr v = (*p)[i];
    if (v->typeName() == "tuple") {
      TuplePtr t = Val_Tuple::cast(v);
      for (unsigned int j = 0; j < t->size(); j++) {
	bytes += (*t)[j]->size();
      }
    } else {
      bytes += (*p)[i]->size();
    }
  }


  boost::posix_time::ptime now_ts;
  
  getTime(now_ts);
  ostringstream b;
  b << "Print[" << _prefix
    << ", "
    << boost::posix_time::to_simple_string(now_ts)
    << "]:  [" << (int) bytes << ", " << p->toString() << "]\n";
  
  if (_output != NULL) {
    fprintf(_output, "%s", b.str().c_str()); 
  } else {
    warn << b.str();
  }
  fflush(_output);
  return p;
}
